// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
/* 
 * Copyright (C) 2011 Micha Hersch,
 *
 * Permission is granted to copy, distribute, and/or modify this program
 * under the terms of the GNU General Public License, version 2 or any
 * later version published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details
 */


#ifndef __ICP_H__
#define __ICP_H__


#include "RigidTransfo.h"

class RigidTransfoBuffer{
 protected:
  RigidTransfo *buffer;
  int size;
  int nb;
  int top;
  int bottom;

 public:
  RigidTransfoBuffer(int size=50);
  int addTransfo(RigidTransfo& rt);
  int getNbElements(){return nb;};
  int getSize(){return size;}
  float getSlope();
};


/**
 * @brief class for doing ICP using the algorithm described in
 * "Iterative Estimation of Rigid-Body Transformation - Application to
 * Robust Object Tracking and Iterative Closest Point", Journal of Mathematical
 * Imaging and Vision, 2011. 
 */
class ICP{

 protected:
  RigidTransfo cand;
  RigidTransfoBuffer trace;
  int ndata;//number of data points
  float *data;
  int ntdata; //number of transformed data points
  float *tdata;
  float *cdata;
  CVector3_t data_center;
  int show_traj;


 public:
  ICP();
  ~ICP();
  float *generate_data(int n);
  void center_data(float *data,int n ,CVector3_t mean);
  void translate_data(float *data,int n,CVector3_t trans);
  float *read_data_from_file(const char *filename, int& n);
 /** 
   * @brief launches icp
   * @param maxit maximal number of iterations 
   */
  void random_transfo(RigidTransfo& rt,float max_angle=pi);
  float *transform_data(RigidTransfo& rt);
  void print_data(float *data, int n, const char *fname);
  /** 
   * @brief launches icp
   * @param maxit maximal number of iterations 
   */
  int perform_icp(int maxit);
  void decenter_transform();
  void init();
  void icp_from_files(char *fname1, char *fname2, int maxit);
  void set_verbose(){show_traj=1;};
  void generate_data_files(const char *fname1, const char *fname2, int nb);
 protected:
  int find_closest_point(float *point, float *data, int n);

};


#endif
