// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
/* 
 * Copyright (C) 2011 Micha Hersch
 *
 * Permission is granted to copy, distribute, and/or modify this program
 * under the terms of the GNU General Public License, version 2 or any
 * later version published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details
 */
#include "RigidTransfo.h"



float RigidTransfo::Update(CVector3_t v, CVector3_t v_tr){

  CVector3_t y,v1,dbeta,dtrans;
  CMatrix3_t B,C,m1; 
  float beta_len,dbeta_len;
  
  ComputeMatrixC(v,C);
  ComputeMatrixB(v,B);

  v_cross(beta,v,v1);//beta x v
  v_mult(v1,beta,m1);//(beta x v)*beta'
  if(alpha>epsilon){
    m_rescale(-2.0/alpha,m1,m1);//2/alpha * (beta x v)*beta'
    m_rescale(2*alpha,B,B);// 2*alpha*B
    m_add(m1,B,m1);// 
    m_sub(m1,C,m1); 
  }

  Transform(v,y);
  v_sub(v_tr,y,v1); // (y-RvR)

  v_transform_normal2(v1,m1,dbeta);
  // fixed increment size
  dbeta_len = v_length(dbeta);
  if(abs(dbeta_len)>epsilon){
    v_scale(dbeta,eps_b/dbeta_len,dbeta);
  }

  v_add(beta,dbeta,beta);//v_add

 
  while(v_length(beta)>(1+1e-2)){
    v_scale(dbeta,0.5,dbeta);
    v_sub(beta,dbeta,beta);   
  }

  beta_len = v_length(beta);
 if(beta_len>1 && beta_len<(1+1e-2)){  //closing the manifold
    v_scale(beta,beta_len-2,beta);
  }

 //for numerical reason 
  if(v_squ_length(beta)>1){
	  alpha=0;
  }else{
    alpha = sqrt(1-v_squ_length(beta));
  }  
  v_scale(v1,eps_tr,dtrans);
  v_add(trans,dtrans,trans); //
  return v_length(v1);
}

int RigidTransfo::SetTransfo(CVector3_t new_beta, CVector3_t transl){
  float sq_len = v_squ_length(new_beta);
  if(sq_len>1){
    return 0;
  } 
  alpha = sqrt(1-sq_len);
  v_copy(new_beta,beta);
  v_copy(transl,trans);
  return 1;
}


void RigidTransfo::SetRate(float n_eps_rot,float n_eps_tr){
  Rotation::SetRate(n_eps_rot);
  Translation::SetRate(n_eps_tr);
}

void RigidTransfo::ScaleRate(float factor){
  Rotation::ScaleRate(factor);
  Translation::ScaleRate(factor);
}


void RigidTransfo::Transform(const CVector3_t in,CVector3_t out){
  CVector3_t v;
  Rotation::Transform(in,v);
  Translation::Transform(v,out);
 
}



void RigidTransfo::InverseTransform(const CVector3_t in,CVector3_t out){
  CVector3_t v;
  Translation::InverseTransform(in,v);
  Rotation::InverseTransform(v,out);
}




/**
 * not the inverse transformation, same order but reverse
 */
void RigidTransfo::ReverseTransform(const CVector3_t in,CVector3_t out){
  CVector3_t v;
  Rotation::InverseTransform(in,v);
  Translation::InverseTransform(v,out);

}


void RigidTransfo::Invert(){
  CVector3_t tr;
  Rotation::Invert();
  Rotation::Transform(trans,tr);
  v_scale(tr,-1,trans);
}

RigidTransfo& RigidTransfo::operator*(const RigidTransfo& rt1){
  CVector3_t v;
  //new translation
  Rotation::Transform(rt1.GetTranslation(),v);
  v_add(v,trans,trans);
  (Rotation&)*this = ((Rotation&)(*this))*(Rotation&)rt1;
  return *this;
}

ostream& operator<<(ostream& out, const RigidTransfo& rt){
  out<<(const Rotation&)(rt)<<" ";
  out<<(const Translation&)(rt);
  return out;
}

istream& operator>>(istream& in, RigidTransfo& rt){
  char c;
  in>>(Rotation&)(rt)>>c;
  in>>(Translation&)(rt);
  return in;
} 


